from datetime import datetime
from typing import Optional

from pydantic import BaseModel


class AgentRun(BaseModel):
    """
    Agent run that records will be associated with.
    """
    run_id: int  # ID of the run (Empower/Fabric ID/Process Run ID)

    run_date: Optional[datetime] = None  # Date of the run

    id: Optional[str] = None  # Generated by the Reporting database (Do not manually specify)
